@@ -228,7 +228,7 @@ class HSTasNet(Module):
             nn.Linear(spec_dim_input, dim)
         )
 
-        self.to_spec_mask = nn.Sequential(
+        self.to_spec_masks = nn.Sequential(
             nn.Linear(dim, spec_dim_input * num_sources),
             Rearrange('b n (s f t) -> (b s) f n t', s = audio_channels, t = num_sources)
         )
@@ -245,12 +245,20 @@ class HSTasNet(Module):
         )
 
         self.to_waveform_masks = nn.Sequential(
-            nn.Linear(dim, num_sources * num_basis, bias = False),
+            nn.Linear(dim, num_sources * num_basis),
             Rearrange('... (t basis) -> ... basis t', t = num_sources)
         )
 
         self.conv_decode = ConvTranspose1DWithHannWindow(num_basis, audio_channels, segment_len, stride = overlap_len)
 
+        # init mask to identity
+
+        nn.init.zeros_(self.to_spec_masks[0].weight)
+        nn.init.constant_(self.to_spec_masks[0].bias, 1.)
+
+        nn.init.zeros_(self.to_waveform_masks[0].weight)
+        nn.init.constant_(self.to_waveform_masks[0].bias, 1.)
+
         # they do a single layer of lstm in their "small" variant
 
         self.small = small
@@ -285,6 +293,21 @@ class HSTasNet(Module):
 
     # saving and loading
 
+    def save_tensor_to_file(
+        self,
+        output_file: str | Path,
+        audio_tensor: Tensor,
+        overwrite = False
+    ):
+        if isinstance(output_file, str):
+            output_file = Path(output_file)
+
+        assert not exists(output_file) or overwrite, f'{str(output_file)} already exists, set `overwrite = True` if you wish to overwrite'
+
+        torchaudio.save(str(output_file), audio_tensor.cpu(), sample_rate = self.sample_rate)
+
+        print(f'audio saved to {str(output_file)}')
+
     def save(self, path, overwrite = True):
         path = Path(path)
         assert overwrite or not path.exists()
@@ -461,11 +484,7 @@ class HSTasNet(Module):
 
         assert output_file != input_file
 
-        assert not output_file.exists() or overwrite, f'{str(output_file)} already exists, set `overwrite = True` if you wish to overwrite'
-
-        torchaudio.save(str(output_file), audio_tensor.cpu(), sample_rate = self.sample_rate)
-
-        print(f'separated audio saved to {str(output_file)}')
+        self.save_tensor_to_file(str(output_file), audio_tensor.cpu(), overwrite = overwrite)
 
     def forward(
         self,
@@ -574,7 +593,7 @@ class HSTasNet(Module):
 
         # spec mask
 
-        spec_mask = self.to_spec_mask(spec).softmax(dim = -1)
+        spec_mask = self.to_spec_masks(spec)
 
         magnitude, phase = complex_spec.abs(), complex_spec.angle()
 
@@ -590,7 +609,7 @@ class HSTasNet(Module):
 
         # waveform mask
 
-        waveform_mask = self.to_waveform_masks(waveform).softmax(dim = -1)
+        waveform_mask = self.to_waveform_masks(waveform)
 
         basis_per_source = multiply('b basis n, b n basis t -> (b t) basis n', basis, waveform_mask)
 
