@@ -8,10 +8,18 @@ from torch.optim import Adam
 from torch.optim.lr_scheduler import StepLR
 from torch.utils.data import Dataset, DataLoader
 
+import numpy as np
+
 from accelerate import Accelerator
 
 from hs_tasnet.hs_tasnet import HSTasNet
 
+from ema_pytorch import EMA
+
+from einops import rearrange
+
+from musdb import DB as MusDB
+
 # functions
 
 def exists(v):
@@ -28,21 +36,51 @@ def not_improved_last_n_steps(losses, steps):
 
     return (last_n_losses[1:] <= last_n_losses[:-1]).all().item()
 
+# wrap the musdb MultiTrack if detected
+
+class MusDBDataset(Dataset):
+    def __init__(
+        self,
+        musdb_data: MultiTrack
+    ):
+        self.musdb_data = musdb_data
+
+    def __len__(self):
+        return len(self.musdb_data)
+
+    def __getitem__(self, idx):
+        sample = self.musdb_data[idx]
+
+        audio = rearrange(sample.audio, 'n s -> s n')
+
+        targets = rearrange(sample.stems[1:], 't n s -> t s n') # the first one is the entire mixture
+
+        audio = audio.astype(np.float32)
+
+        targets = targets.astype(np.float32)
+
+        return audio, targets
+
 # classes
 
 class Trainer(Module):
     def __init__(
         self,
         model: HSTasNet,
-        dataset: Dataset,
+        dataset: Dataset | MusDB,
         eval_dataset: Dataset | None = None,
         optim_klass = Adam,
         batch_size = 128,
+        grad_accum_every = 1,
         learning_rate = 3e-4,
         max_epochs = 10,
+        max_steps = None,
         accelerate_kwargs: dict = dict(),
         optimizer_kwargs: dict = dict(),
         cpu = False,
+        use_ema = True,
+        ema_decay = 0.995,
+        ema_kwargs: dict = dict(),
         checkpoint_every = 1,
         checkpoint_folder = './checkpoints',
         decay_lr_factor = 0.5,
@@ -55,6 +93,8 @@ class Trainer(Module):
 
         self.max_epochs = max_epochs
 
+        self.max_steps = max_steps
+
         # saving
 
         self.checkpoint_every = checkpoint_every
@@ -70,7 +110,13 @@ class Trainer(Module):
             **optimizer_kwargs
         )
 
-        # data
+        # dataset
+
+        if isinstance(dataset, MusDB):
+            # give musdb special treatment
+            dataset = MusDBDataset(dataset)
+
+        # dataloader
 
         dataloader = DataLoader(dataset, batch_size = batch_size, drop_last = True, shuffle = True)
 
@@ -82,6 +128,7 @@ class Trainer(Module):
 
         self.accelerator = Accelerator(
             cpu = cpu,
+            gradient_accumulation_steps = grad_accum_every,
             **accelerate_kwargs
         )
 
@@ -91,6 +138,13 @@ class Trainer(Module):
 
         self.decay_lr_if_not_improved_steps = decay_lr_if_not_improved_steps
 
+        # setup ema on main process
+
+        self.use_ema = use_ema
+
+        if use_ema:
+            self.ema_model = EMA(model, beta = ema_decay, **ema_kwargs)
+
         # preparing
 
         (
@@ -131,6 +185,13 @@ class Trainer(Module):
     def unwrapped_model(self):
         return self.accelerator.unwrap_model(self.model)
 
+    @property
+    def is_main(self):
+        return self.accelerator.is_main_process
+
+    def wait(self):
+        return self.accelerator.wait_for_everyone()
+
     def print(self, *args):
         return self.accelerator.print(*args)
 
@@ -139,72 +200,104 @@ class Trainer(Module):
 
     def forward(self):
 
+        exceeds_max_step = False
         past_eval_losses = [] # for learning rate decay and early stopping detection
 
         for epoch in range(self.max_epochs):
 
+            self.model.train()
+
             # training steps
 
             for audio, targets in self.dataloader:
-                loss = self.model(audio, targets = targets)
 
-                self.print(f'[{epoch}] loss: {loss.item():.3f}')
+                with self.accelerator.accumulate(self.model):
 
-                self.log(loss = loss)
+                    loss = self.model(audio, targets = targets)
 
-                self.accelerator.backward(loss)
+                    self.accelerator.backward(loss)
 
-                self.optimizer.step()
-                self.optimizer.zero_grad()
+                    self.print(f'[{epoch}] loss: {loss.item():.3f}')
 
-            if not self.needs_eval:
-                continue
+                    self.log(loss = loss)
 
-            self.accelerator.wait_for_everyone()
+                    self.optimizer.step()
+                    self.optimizer.zero_grad()
 
-            # evaluation at the end of each epoch
+                # max steps
 
-            eval_losses = []
+                self.step.add_(1)
 
-            for eval_audio, eval_targets in self.eval_dataloader:
+                exceeds_max_step = exists(self.max_steps) and self.step.item() >= self.max_steps
 
-                self.model.eval()
+                if exceeds_max_step:
+                    break
 
-                with torch.no_grad():
-                    eval_loss = self.model(audio, targets = targets)
-                    eval_losses.append(eval_loss)
+            # update ema
 
-                avg_eval_loss = stack(eval_losses).mean()
-                past_eval_losses.append(avg_eval_loss)
+            self.wait()
 
-            self.print(f'[{epoch}] eval loss: {avg_eval_loss.item():.3f}')
+            if self.use_ema and self.is_main:
+                self.ema_model.update()
 
-            self.log(loss = avg_eval_loss)
+            # maybe eval
+
+            self.wait()
+
+            if self.needs_eval:
+
+                # evaluation at the end of each epoch
+
+                eval_losses = []
+
+                for eval_audio, eval_targets in self.eval_dataloader:
+
+                    self.model.eval()
+
+                    with torch.no_grad():
+                        eval_loss = self.model(audio, targets = targets)
+                        eval_losses.append(eval_loss)
+
+                    avg_eval_loss = stack(eval_losses).mean()
+                    past_eval_losses.append(avg_eval_loss)
+
+                self.print(f'[{epoch}] eval loss: {avg_eval_loss.item():.3f}')
+
+                self.log(loss = avg_eval_loss)
 
             # maybe save
 
+            self.wait()
+
             if (
                 divisible_by(epoch + 1, self.checkpoint_every) and
-                self.accelerator.is_main_process
+                self.is_main
             ):
                 checkpoint_index = (epoch + 1) // self.checkpoint_every
                 self.unwrapped_model.save(self.checkpoint_folder / f'hs-tasnet.ckpt.{checkpoint_index}.pt')
 
-            self.accelerator.wait_for_everyone()
+                if self.use_ema:
+                    self.ema_model.ema_model.save(self.checkpoint_folder /f'hs_tasnet.ema.ckpt.{checkpoint_index}.pt') # save ema
 
-            # stack validation losses for all epochs
+            # determine lr decay and early stopping based on eval
 
-            last_n_eval_losses = stack(past_eval_losses[-self.early_stop_steps:])
+            if self.needs_eval:
+                # stack validation losses for all epochs
 
-            # decay lr if criteria met
+                last_n_eval_losses = stack(past_eval_losses)
 
-            if not_improved_last_n_steps(last_n_eval_losses, self.decay_lr_if_not_improved_steps):
-                self.scheduler.step()
+                # decay lr if criteria met
 
-            # early stop if criteria met
+                if not_improved_last_n_steps(last_n_eval_losses, self.decay_lr_if_not_improved_steps):
+                    self.scheduler.step()
 
-            if not_improved_last_n_steps(last_n_eval_losses, self.early_stop_steps):
-                self.print(f'early stopping at epoch {epoch} since last three eval losses have not improved: {last_n_eval_losses}')
+                # early stop if criteria met
+
+                if not_improved_last_n_steps(last_n_eval_losses, self.early_stop_steps):
+                    self.print(f'early stopping at epoch {epoch} since last three eval losses have not improved: {last_n_eval_losses}')
+                    break
+
+            if exceeds_max_step:
                 break
 
-            self.step.add_(1)
+        self.print(f'training complete')
