@@ -44,8 +44,10 @@ def test_model(
     assert out3.shape == (*prec_shape, 512)
 
 @param('with_eval', (False, True))
+@param('with_ema', (False, True))
 def test_trainer(
-    with_eval
+    with_eval,
+    with_ema
 ):
     from hs_tasnet.hs_tasnet import HSTasNet
     from hs_tasnet.trainer import Trainer
@@ -79,7 +81,8 @@ def test_trainer(
         batch_size = 4,
         max_epochs = 3,
         checkpoint_every = 1,
-        cpu = True
+        cpu = True,
+        use_ema = with_ema
     )
 
     trainer()
@@ -102,3 +105,15 @@ def test_audio_processing():
     )
 
     model.process_audio_file('./tests/test.mp3', [0, 2], overwrite = True)
+
+def test_musdb():
+    import musdb
+    from hs_tasnet.hs_tasnet import HSTasNet
+    from hs_tasnet.trainer import Trainer
+
+    mus = musdb.DB(download = True)
+
+    model = HSTasNet(stereo = True)
+
+    trainer = Trainer(model, dataset = mus, batch_size = 4, cpu = True, max_steps = 2)
+    trainer()
